
#include "convert.h"


void debug_flush(void) {
  eassert(!fflush(debug));
  eassert(!ferror(debug));
}


const char *get_vardir(void) { return "."; }

static enum {
  mf_findwindow=    01,
  mf_screenshot=    02,
  mf_analyse=       04,
  
  mode_findwindow=  01,
  mode_screenshot=  03,
  mode_analyse=     04,

  mode_all=         07,
} o_mode= mode_all;

static char *o_screenshots_fn;
static int o_single_page, o_quiet;

FILE *screenshots_file;

int main(int argc, char **argv) {
  const char *arg;
  int r;

  while ((arg=*++argv)) {
    if (!strcmp(arg,"--find-window-only"))
      o_mode= mode_findwindow;
    else if (!strcmp(arg,"--screenshot-only"))
      o_mode= mode_screenshot;
    else if (!strcmp(arg,"--analyse-only"))
      o_mode= mode_analyse;
    else if (!strcmp(arg,"--single-page"))
      o_single_page= 1;
    else if (!strcmp(arg,"--quiet"))
      o_quiet= 1;
    else if (!strcmp(arg,"--screenshots-file"))
      eassert( o_screenshots_fn= *++argv );
#define DF(f)					\
    else if (!strcmp(arg,"-D" #f))		\
      debug_flags |= dbg_##f;
    DEBUG_FLAG_LIST
#undef DF
    else if (!strcmp(arg,"--window-id")) {
      char *ep;
      eassert((arg=*++argv));
      unsigned long windowid= strtoul(arg,&ep,0);
      eassert(!*ep);
      set_yppclient_window(windowid);
    } else
      eassert(!"bad option");
  }
  
  if (!o_screenshots_fn) {
    r= asprintf(&o_screenshots_fn,"%s/#pages#.ppm",get_vardir());
    eassert(r>=0);  eassert(o_screenshots_fn);
  }

  if (o_mode & mf_findwindow) {
    screenshot_startup();
    find_yppclient_window();
  }
  if (o_mode & mf_screenshot) {
    screenshots_file= fopen(o_screenshots_fn, "w"); eassert(screenshots_file);
    if (o_single_page)
      take_one_screenshot();
    else
      take_screenshots();
  } else {
    screenshots_file= fopen(o_screenshots_fn, "r"); eassert(screenshots_file);
    read_screenshots();
  }
  if (o_mode & mf_analyse) {
    analyse();
    //output_tsv();
  }
  return 0;
}




DEFINE_VWRAPPERF(, progress)
DEFINE_VWRAPPERF(, progress_log)

static int last_progress_len;
     
void vprogress(const char *fmt, va_list al) {
  int r;
  
  if (o_quiet) return;
  if (!isatty(2)) return;
  
  if (last_progress_len)
    putc('\r',stderr);

  r= vfprintf(stderr,fmt,al);
  eassert(r>=0);

  if (r < last_progress_len) {
    fprintf(stderr,"%*s", last_progress_len - r, "");
    if (!r) putc('\r', stderr);
    else while (last_progress_len-- > r) putc('\b',stderr);
  }
  last_progress_len= r;
  fflush(stderr);
}
   
void vprogress_log(const char *fmt, va_list al) {
  if (o_quiet) return;
  
  progress("");
  vfprintf(stderr,fmt,al);
  putc('\n',stderr);
  fflush(stderr);
}
