/*
 * ocr.c forms a mostly-self-contained bit
 * so we put its declarations in this separate file
 */

#ifndef OCR_H
#define OCR_H


#define _GNU_SOURCE

#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>
#include <stdint.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <inttypes.h>

#include <sys/types.h>
#include <sys/wait.h>


typedef struct {
  int w,h;
  char d[];
} CanonImage;

typedef uint32_t Pixcol;
#define PSPIXCOL(priscan) priscan##32

typedef struct {
  const char *s; /* valid until next call to ocr() */
  int l,r; /* column numbers */
  unsigned ctxmap; /* match context index */
} OcrResultGlyph;


typedef const struct OcrCellTypeInfo *OcrCellType;
extern const struct OcrCellTypeInfo ocr_celltype_text;
extern const struct OcrCellTypeInfo ocr_celltype_number;


typedef struct OcrReader OcrReader;
OcrReader *ocr_init(int h);

OcrResultGlyph *ocr(OcrReader *rd, OcrCellType, int w, Pixcol cols[]);
  /* return value is array terminated by {0,-1,-1}
   * array is valid until next call to ocr()
   */


/*----- debugging arrangements, rather contingent -----*/

#define DEBUG_FLAG_LIST				\
   DF(findypp)					\
   DF(pages)					\
   DF(rect)					\
   DF(ocr)					\
   DF(callout)

enum {
#define DF(f) dbg__shift_##f,
  DEBUG_FLAG_LIST
#undef DF
};
enum {
#define DF(f) dbg_##f = 1 << dbg__shift_##f,
  DEBUG_FLAG_LIST
#undef DF
};

unsigned debug_flags;

#define DEBUGP(f) (!!(debug_flags & dbg_##f))

void debug_flush(void);

#define eassert assert
#define debug stderr

const char *get_vardir(void);

#define DEFINE_VWRAPPERF(decls, funcf)					\
  decls void funcf(const char *fmt, ...) {				\
    va_list al;  va_start(al,fmt);  v##funcf(fmt,al);  va_end(al);	\
  }

#define DEBUG_DEFINE_SOME_DEBUGF(fl,funcf)				\
  static void v##funcf(const char *fmt, va_list al) {			\
    if (DEBUGP(fl))							\
      vfprintf(debug,fmt,al);						\
  }									\
  DEFINE_VWRAPPERF(static, funcf)

#define DEBUG_DEFINE_DEBUGF(fl) DEBUG_DEFINE_SOME_DEBUGF(fl,debugf)

#endif /*OCR_H*/
